import "Turbine";
import "Turbine.Debug";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "DhorPlugins.Extensions";
import "DhorPlugins.Utils.BitOps";


--[[ This is the main window for the Travel Window UI	                      ]]--
--[[ It handles the loading and saving the settings,	                      ]]--
--[[ creating the lists of travel shortcuts, creates	                      ]]--
--[[ and handles the context menu.						                         ]]--


MainWindow = class( Turbine.UI.Extensions.SimpleWindow );

function MainWindow:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );	
	
   -- get the player class and race
   player = Turbine.Gameplay.LocalPlayer.GetInstance();
   playerClass = player:GetClass();
   
   -- quit if the class is not a hunter                        
   if(playerClass ~= Turbine.Gameplay.Class.Hunter) then
      Turbine.Shell.WriteLine("This plugin will only work for hunters.");
      return;
   end
   	
	settings = { }; 	
	self.minWidth = 21;
	self.minHeight = 21;
	self.options = nil;
	
	-- load the player saved settings
	self:LoadSettings();
	self.bgColor = settings.backColor;

	-- configure the main window
	self:SetPosition(settings.positionX,settings.positionY);	
	self:SetSize( settings.width, settings.height );
	self:SetOpacity( settings.mainMinOpacity );
	self:SetZOrder(1);
	self:SetBackColor(self.bgColor);
	self:SetVisible(true); 
   
   -- create a control for resizing the window
   self.sizeControl = Turbine.UI.Control();
   self.sizeControl:SetParent(self);
   self.sizeControl:SetZOrder(100);
   self.sizeControl:SetSize(20,20);
   self.sizeControl:SetBackColorBlendMode(Turbine.UI.BlendMode.Screen)
   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
   self.sizeControl:SetPosition(self:GetWidth() - self.sizeControl:GetWidth(),
                            self:GetHeight() - self.sizeControl:GetHeight());   
   
   -- add the labels for the focus level   
   self.labels = { };
   for i = 1,9,1 do
      self.labels[i] = Turbine.UI.Label();		
		self.labels[i]:SetParent( self );
		self.labels[i]:SetBackColor(settings.focusColor);
		self.labels[i]:SetZOrder(50);
		self.labels[i]:SetMouseVisible(false);
   end  
   
   -- save the player's combat states for managing hiding the window
   -- when the player enters combat
   self.previousCombatState = false;
   self.wasOpenBeforeCombat = false;
   
	-- create a single context menu to use on all panels
	menu = Turbine.UI.ContextMenu(self);
	self.options = Turbine.UI.MenuItem(optionsString);
	menuItems = menu:GetItems();
	menuItems:Add(self.options);
	
	-- open the option window when clicked
	menuItems:Get(1).Click = function(sender, args)
	   self:OpenOptions();
	end
		
	-- hunter attribute
	self.hunterFocus = player:GetClassAttributes();
	
   -- timers for events
	self.lastTime = 0;
   self.lastMove = 0;
   
   -- track the hidden state of the UI, manage previous states for window and
   -- the button
   self.hidden = false;
   self.currentVisState = self:IsVisible();
   
   self:UpdateLabels();
   self:UpdateFocus(self.hunterFocus:GetFocus());     
   
--[[ Event Handlers ]]--

	-- make sure we listen for updates
	self:SetWantsUpdates(true);
	self:SetWantsKeyEvents(true);
	
   local isMoving = false;
	local x = 0;
	local y = 0;

   -- handle window mouse down event (not quickslot or button events)
	self.MouseDown = function(sender, args )
	   -- if its the left button, prepare to move the window by setting
	   -- the flag, and storing the current mouse position
		if (args.Button == Turbine.UI.MouseButton.Left) then
			isMoving = true;
			x = args.X;
			y = args.Y;
		else
		   menu:ShowMenu();
		end
	end

   -- handle window mouse up event
	self.MouseUp = function(sender, args )
	   -- check if its the left mouse button, otherwise ignore
		if (args.Button == Turbine.UI.MouseButton.Left) then
		
		   -- clear the flag
			isMoving = false;

         -- save the current position settings
			local one,two = self:GetPosition();
			settings.positionX = one;
			settings.positionY = two;
			self:SaveSettings();
		end
	end

   -- handle window mouse move event
	self.MouseMove = function(sender, args )
	   -- if the moving flag is set, move the window
		if (isMoving) then
		   -- get the old position of the window
			local oldX,oldY = self:GetPosition();
			
			-- update the window position based on the previous mouse location,
			-- previous window position, and the current mouse position
			self:SetPosition(oldX + args.X - x,oldY + args.Y - y);	
		end
	end
	
	
	
	
	-- check if our position has changed, and save the settings if so
	self.PositionChanged = function(sender, args )
		one,two = self:GetPosition();
		settings.positionX = one;
		settings.positionY = two;
		self:SaveSettings();
	end
	
   -- manage hiding the UI  with F12
   self.KeyDown = function( sender, args)
      if(args.Action == 268435635) then
         if(self.hidden == false) then
             self.currentVisState = self:IsVisible();
             self.hidden = true;
             self:SetVisible(false);
             self.ToggleButton:SetVisible(false);
         else
             self.hidden = false;
             self:SetVisible(self.currentVisState);
             self.ToggleButton:SetVisible(settings.showButton == 1);
         end
      end
   end
   
   -- check if the player's focus level changes
   self.hunterFocus.FocusChanged = function(sender, args)
      self:UpdateFocus(self.hunterFocus:GetFocus());
   end
   
   -- check if the combat state of the player changed, to adjust the opacity
   -- of the window
   player.InCombatChanged = function(sender, args)

      if(player:IsInCombat()) then
         self:SetOpacity(settings.mainMaxOpacity)
      else
         self:SetOpacity(settings.mainMinOpacity)
	   end
   end
	
   -- go to full opacity if mouse is over	
   self.MouseEnter = function(sender, args)
      self:SetBackColor(self.bgColor);
   end
   
   -- go to low opacity when mosue is not over
   self.MouseLeave = function(sender, args)
      self:SetBackColor(self.bgColor);
   end 

	-- change the background color of the resize button on entry to help
	-- show exactly where the button is
	self.sizeControl.MouseEnter = function(sender,args)	      	  
     self.sizeControl:SetBackColor(Turbine.UI.Color(0.9,1,1,1));
	end
	
	-- return background color to normal on mouse leave
	self.sizeControl.MouseLeave = function(sender,args)
	   self.sizeControl:SetBackColor(Turbine.UI.Color(0.01,1,1,1));
	end
	
	-- check if someone is going to start dragging the resize control
	self.sizeControl.MouseDown = function(sender,args)
	
	  -- check if resize has been disabled
	  if(self.disableResize) then
	     return;
	  end  
	
	  -- set start location of resize and enable dragging
	  sender.dragStartX = args.X;
	  sender.dragStartY = args.Y;
	  sender.dragging = true;
	end
	
	self.sizeControl.MouseUp = function(sender,args)
	  -- disable dragging, then update settings
	  sender.dragging = false;	  
	  
	  settings.width = self:GetWidth();
	  settings.height = self:GetHeight();	  
	  self:SaveSettings();
	end
	
	-- perform the resize
	self.sizeControl.MouseMove = function(sender,args)	
	  	 local width, height = self:GetSize();
      
         -- if resize dragging, then resize the window 		
		 if ( sender.dragging ) then
			   self:SetSize( width + ( args.X - sender.dragStartX ), 
                        height + ( args.Y - sender.dragStartY ) );
		 end
		 
		 -- update the size of the window
		 self:UpdateSize();      
	end                       
end

function MainWindow:UpdateSize()
         -- check that the window is not smaller than min width	
		 if (self:GetWidth() < self.minWidth) then
		       self:SetWidth(self.minWidth);
		 end
		
		 -- check that the window is not smaller than min height
		 if (self:GetHeight() < self.minHeight) then
		       self:SetHeight(self.minHeight);
       end
      
       -- set the new location of the resize control
		 self.sizeControl:SetPosition( 
                           self:GetWidth() - self.sizeControl:GetWidth(), 
                           self:GetHeight() - self.sizeControl:GetHeight() );
      
                           
       self:SaveSettings();
       self:UpdateLabels();
end  

function MainWindow:LoadSettings()
	-- load the self.settings.  If a value is not available, set a default value
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, 
                                                "HunterFocusSettings" );

	if ( type( settings ) ~= "table" ) then
		settings = { };
	end
	
	if ( not settings.width) then
	   settings.width = 200;
	end
	
	if (not settings.height) then
	   settings.height = 50;
	end

	if ( not settings.positionX ) then
		settings.positionX = 200; 
	end

	if ( not settings.positionY ) then
		settings.positionY = 200;
	end
	
	if ( not settings.hideOnStart ) then
	   settings.hideOnStart = 0;
	end
   	
	if ( not settings.hideOnCombat ) then
	   settings.hideOnCombat = 0;
	end
	
	if ( not settings.horizontal ) then
	   settings.horizontal = 1;
	end

	if ( not settings.mainMaxOpacity ) then
	   settings.mainMaxOpacity = 1;
	end
	
	if ( not settings.mainMinOpacity ) then
	  settings.mainMinOpacity = 0.5;
	end
	
	if ( not settings.focusColor ) then
	  settings.focusColor = Turbine.UI.Color(1,0.9,0.1,0.1);
	end
	
	if ( not settings.backColor ) then
	  settings.backColor = Turbine.UI.Color(1,1,1,1);
	end
	
	settings.focusColor = Turbine.UI.Color(settings.focusColor.A,settings.focusColor.R,settings.focusColor.G,settings.focusColor.B);
	settings.backColor = Turbine.UI.Color(settings.backColor.A,settings.backColor.R,settings.backColor.G,settings.backColor.B);

end

function MainWindow:SaveSettings()
	-- save the settings
	Turbine.PluginData.Save( Turbine.DataScope.Character,
                                  "HunterFocusSettings", settings );
end

function MainWindow:UpdateLabels()

   -- loop through the labels and adjust the size and position of each
   for i = 1,9,1 do
      if(settings.horizontal == 1) then
         local labelWidth = (self:GetWidth() - 10)/9;    
         self.labels[i]:SetSize(labelWidth,self:GetHeight()-2);
         self.labels[i]:SetPosition((i-1) * labelWidth + i, 1);
      else
         local labelHeight = (self:GetHeight() - 10)/9;    
         self.labels[i]:SetSize(self:GetWidth() - 2,labelHeight);
         self.labels[i]:SetPosition(1,self:GetHeight() - ((i-1) * labelHeight + i) - labelHeight);
      end
      
      self.labels[i]:SetBackColor(settings.focusColor);        	
   end                                                 
   
   self:SetBackColor(settings.backColor);       
end

function MainWindow:UpdateFocus(level)

   -- loop through all the labels and set the visibility based on the
   -- current level of focus
   
   for i = 1,9,1 do
     if(i <= level) then
       self.labels[i]:SetVisible(true);
     else
       self.labels[i]:SetVisible(false);
     end
   end   
end   

-- for debug purposes
function MainWindow:DoDump()
	Turbine.Debug.Table.Dump(DhorPlugins.HunterFocus);
end

-- simple function to open the options window
function MainWindow:OpenOptions()
    self.options = DhorPlugins.HunterFocus.OptionsWindow(self);
end

-- function to close the options window if it exists
function MainWindow:CloseOptions()
   if(self.options ~= nil) then
      self.options:SetVisible(false);
   end
   self.options = nil;
end

function MainWindow:UpdateOpacity()
   self:SetOpacity(settings.mainMinOpacity);
end
